@@ -7,7 +7,7 @@
  *   http://www.openbsd.org/.
  *
  * ----------------------------------------------------------------------- */
-/* $Id: config.h,v 1.9 2001/11/14 04:16:23 hpa Exp $ */
+/* $Id: config.h,v 1.12 2001/11/17 01:55:08 hpa Exp $ */
 
 /*
  * config.h
@@ -57,7 +57,9 @@
 #endif 
 
 #ifdef HAVE_INTTYPES_H
+#ifdef INTTYPES_H_IS_SANE
 #include <inttypes.h>
+#endif
 #else
 #ifdef HAVE_STDINT_H
 #include <stdint.h>
@@ -68,7 +70,28 @@
 #include <unistd.h>
 #endif
 
+#ifdef HAVE_SETJMP_H
 #include <setjmp.h>
+#endif
+
+#ifdef TIME_WITH_SYS_TIME
+#include <sys/time.h>
+#include <time.h>
+#else
+#if HAVE_SYS_TIME_H
+#include <sys/time.h>
+#else
+#include <time.h>
+#endif
+#endif
+
+#ifdef HAVE_GRP_H
+#include <grp.h>
+#endif
+
+#include <errno.h>
+#include <signal.h>
+
 #include <sys/socket.h>
 
 /* If we don't have intmax_t, try creating it */
@@ -93,6 +116,37 @@ typedef unsigned long uintmax_t;
 #endif
 #endif
 
+/* On some version of AIX, <inttypes.h> is buggy to the point of
+   unusability.  We have to use macros here, not typedefs, to override. */
+#ifdef HAVE_INTTYPES_H
+#ifndef INTTYPES_H_IS_SANE
+#undef PRIdMAX
+#undef PRIuMAX
+#undef PRIxMAX
+#undef INTMAX_C
+#undef UINTMAX_C
+#undef HAVE_STRTOUMAX
+
+#ifdef HAVE_LONG_LONG
+#define intmax_t long long
+#define uintmax_t unsigned long long
+#define PRIdMAX	"Ld"
+#define PRIuMAX "Lu"
+#define PRIxMAX "Lx"
+#define INTMAX_C(x)  (x##LL)
+#define UINTMAX_C(x) (x##ULL)
+#else
+#define intmax_t long
+#define uintmax_t unsigned long
+#define PRIdMAX	"ld"
+#define PRIuMAX "lu"
+#define PRIxMAX "lx"
+#define INTMAX_C(x)  (x##L)
+#define UINTMAX_C(x) (x##UL)
+#endif
+#endif
+#endif
+
 /* Even if intmax_t is defined, we may need this (Solaris 8 braindamage) */
 #ifndef HAVE_STRTOUMAX
 #if defined(HAVE_LONG_LONG) && defined(HAVE_STRTOULL)
