@@ -43,7 +43,7 @@ static const char *copyright UNUSED =
 /* static char sccsid[] = "@(#)main.c	8.1 (Berkeley) 6/6/93"; */
 /* static char rcsid[] = "$OpenBSD: main.c,v 1.4 1997/01/17 07:13:30 millert Exp $"; */
 static const char *rcsid UNUSED =
- "tftp-hpa $Id: main.c,v 1.16 2001/11/14 20:51:57 hpa Exp $";
+ "tftp-hpa $Id: main.c,v 1.17 2001/11/16 20:15:27 hpa Exp $";
 #endif /* not lint */
 
 /* Many bug fixes are from Jim Guyton <guyton@rand-unix> */
@@ -51,23 +51,9 @@ static const char *rcsid UNUSED =
 /*
  * TFTP User Program -- Command Interface.
  */
-#include <sys/types.h>
-#include <sys/socket.h>
 #include <sys/file.h>
-
-#include <netinet/in.h>
-
-#include <arpa/inet.h>
-
 #include <ctype.h>
-#include <errno.h>
 #include <netdb.h>
-#include <setjmp.h>
-#include <signal.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <unistd.h>
 #include <fcntl.h>
 #ifdef WITH_READLINE
 #include <readline/readline.h>
