@@ -1,4 +1,4 @@
-/* tftp-hpa: $Id: tftpsubs.c,v 1.6 2001/11/12 23:27:48 hpa Exp $ */
+/* tftp-hpa: $Id: tftpsubs.c,v 1.7 2001/11/16 20:15:27 hpa Exp $ */
 
 /* $OpenBSD: tftpsubs.c,v 1.2 1996/06/26 05:40:36 deraadt Exp $	*/
 /* $NetBSD: tftpsubs.c,v 1.3 1994/12/08 09:51:31 jtc Exp $	*/
@@ -42,7 +42,7 @@
 /* static char sccsid[] = "@(#)tftpsubs.c	8.1 (Berkeley) 6/6/93"; */
 /* static char rcsid[] = "$OpenBSD: tftpsubs.c,v 1.2 1996/06/26 05:40:36 deraadt Exp $"; */
 static const char *rcsid UNUSED =
-"tftp-hpa: $Id: tftpsubs.c,v 1.6 2001/11/12 23:27:48 hpa Exp $";
+"tftp-hpa: $Id: tftpsubs.c,v 1.7 2001/11/16 20:15:27 hpa Exp $";
 #endif /* not lint */
 
 /* Simple minded read-ahead/write-behind subroutines for tftp user and
@@ -56,15 +56,7 @@ static const char *rcsid UNUSED =
 			Jim Guyton 10/85
  */
 
-#include <sys/types.h>
-#include <sys/socket.h>
 #include <sys/ioctl.h>
-#include <sys/time.h>
-#include <netinet/in.h>
-#include <arpa/tftp.h>
-#include <stdio.h>
-#include <string.h>
-#include <unistd.h>
 
 #define PKTSIZE MAX_SEGSIZE+4       /* should be moved to tftp.h */
 
