@@ -1,4 +1,4 @@
-/* tftp-hpa: $Id: tftpd.c,v 1.38 2001/11/15 04:11:39 hpa Exp $ */
+/* tftp-hpa: $Id: tftpd.c,v 1.42 2001/11/20 19:45:12 hpa Exp $ */
 
 /* $OpenBSD: tftpd.c,v 1.13 1999/06/23 17:01:36 deraadt Exp $	*/
 
@@ -45,7 +45,7 @@ static const char *copyright UNUSED =
 /*static char sccsid[] = "from: @(#)tftpd.c	5.13 (Berkeley) 2/26/91";*/
 /*static char rcsid[] = "$OpenBSD: tftpd.c,v 1.13 1999/06/23 17:01:36 deraadt Exp $: tftpd.c,v 1.6 1997/02/16 23:49:21 deraadt Exp $";*/
 static const char *rcsid UNUSED =
-"tftp-hpa $Id: tftpd.c,v 1.38 2001/11/15 04:11:39 hpa Exp $";
+"tftp-hpa $Id: tftpd.c,v 1.42 2001/11/20 19:45:12 hpa Exp $";
 #endif /* not lint */
 
 /*
@@ -54,28 +54,14 @@ static const char *rcsid UNUSED =
  * This version includes many modifications by Jim Guyton <guyton@rand-unix>
  */
 
-#include <sys/types.h>
 #include <sys/ioctl.h>
-#include <sys/stat.h>
-#include <sys/time.h>
 #include <signal.h>
 #include <fcntl.h>
-
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <arpa/tftp.h>
 #include <netdb.h>
-
-#include <setjmp.h>
-#include <syslog.h>
-#include <stdio.h>
-#include <errno.h>
 #include <ctype.h>
-#include <string.h>
-#include <stdlib.h>
 #include <pwd.h>
-#include <unistd.h>
 #include <limits.h>
+#include <syslog.h>
 
 #include "tftpsubs.h"
 #include "recvfrom.h"
@@ -117,6 +103,7 @@ const char **dirs;
 
 int	secure = 0;
 int	cancreate = 0;
+int	unixperms = 0;
 
 int verbosity = 0;
 
@@ -198,11 +185,13 @@ main(int argc, char **argv)
   int standalone = 0;		/* Standalone (listen) mode */
   char *address = NULL;		/* Address to listen to */
   pid_t pid;
+  mode_t my_umask = 0;
+  int spec_umask = 0;
   int c;
   int setrv;
   int waittime = 900;		/* Default time to wait for a connect*/
   const char *user = "nobody";	/* Default user */
-  char *p;
+  char *p, *ep;
 #ifdef WITH_REGEX
   char *rewrite_file = NULL;
 #endif
@@ -214,7 +203,7 @@ main(int argc, char **argv)
   
   openlog(__progname, LOG_PID|LOG_NDELAY, LOG_DAEMON);
   
-  while ((c = getopt(argc, argv, "csvVla:u:r:t:m:")) != -1)
+  while ((c = getopt(argc, argv, "cspvVla:u:U:r:t:m:")) != -1)
     switch (c) {
     case 'c':
       cancreate = 1;
@@ -222,6 +211,9 @@ main(int argc, char **argv)
     case 's':
       secure = 1;
       break;
+    case 'p':
+      unixperms = 1;
+      break;
     case 'l':
       standalone = 1;
       break;
@@ -234,6 +226,14 @@ main(int argc, char **argv)
     case 'u':
       user = optarg;
       break;
+    case 'U':
+      my_umask = strtoul(optarg, &ep, 8);
+      if ( *ep ) {
+	syslog(LOG_ERR, "Invalid umask: %s", optarg);
+	exit(EX_USAGE);
+      }
+      spec_umask = 1;
+      break;
     case 'r':
       for ( opt = options ; opt->o_opt ; opt++ ) {
 	if ( !strcasecmp(optarg, opt->o_opt) ) {
@@ -294,6 +294,9 @@ main(int argc, char **argv)
     syslog(LOG_ERR, "no user %s: %m", user);
     exit(EX_NOUSER);
   }
+
+  if ( spec_umask || !unixperms )
+    umask(my_umask);
   
   if (ioctl(fd, FIONBIO, &on) < 0) {
     syslog(LOG_ERR, "ioctl(FIONBIO): %m");
@@ -310,6 +313,7 @@ main(int argc, char **argv)
     fd = socket(PF_INET, SOCK_DGRAM, 0);
     
     memset(&bindaddr, 0, sizeof bindaddr);
+    bindaddr.sin_family = AF_INET;
     bindaddr.sin_addr.s_addr = INADDR_ANY;
     bindaddr.sin_port = htons(IPPORT_TFTP);
 
@@ -488,13 +492,29 @@ main(int argc, char **argv)
     exit(EX_IOERR);
   }
 
+  /* Set up the supplementary group access list if possible */
+  /* /etc/group still need to be accessible at this point */
+#ifdef HAVE_INITGROUPS
+  setrv = initgroups(user, pw->pw_gid);
+  if ( setrv ) {
+    syslog(LOG_ERR, "cannot set groups for user %s", user);
+    exit(EX_OSERR);
+  }
+#else
+#ifdef HAVE_SETGROUPS
+  if ( setgroups(0, NULL) ) {
+    syslog(LOG_ERR, "cannot clear group list");
+  }
+#endif
+#endif
+
   /* Chroot and drop privileges */
   
   if (secure && chroot(".")) {
     syslog(LOG_ERR, "chroot: %m");
     exit(EX_OSERR);
   }
-  
+
 #ifdef HAVE_SETREGID
   setrv = setregid(pw->pw_gid, pw->pw_gid);
 #else
@@ -793,23 +813,64 @@ do_opt(char *opt, char *val, char **ap)
   return;
 }
 
+#ifdef WITH_REGEX
+
+/*
+ * This is called by the remap engine when it encounters macros such
+ * as \i.  It should write the output in "output" if non-NULL, and
+ * return the length of the output (generated or not).
+ *
+ * Return -1 on failure.
+ */
+int
+rewrite_macros(char macro, char *output);
+
+int
+rewrite_macros(char macro, char *output)
+{
+  char *p;
+
+  switch (macro) {
+  case 'i':
+    p = inet_ntoa(from.sin_addr);
+    if ( output )
+      strcpy(output, p);
+    return strlen(p);
+    
+  case 'x':
+    if ( output )
+      sprintf(output, "%08X",
+	      ntohl(from.sin_addr.s_addr));
+    return 8;
+
+  default:
+    return -1;
+  }
+}
+
 /*
  * Modify the filename, if applicable.  If it returns NULL, deny the access.
  */
 char *
 rewrite_access(char *filename, int mode)
 {
-#ifdef WITH_REGEX
   if ( rewrite_rules ) {
-    char *newname = rewrite_string(filename, rewrite_rules, mode != RRQ);
+    char *newname = rewrite_string(filename, rewrite_rules,
+				   mode != RRQ, rewrite_macros);
     filename = newname;
   }
-#else
-  (void)mode;			/* Suppress unused warning */
-#endif
   return filename;
 }
 
+#else
+char *
+rewrite_access(char *filename, int mode)
+{
+  (void)mode;			/* Avoid warning */
+  return filename;
+}
+#endif
+
 FILE *file;
 /*
  * Validate file access.  Since we
@@ -858,47 +919,53 @@ validate_access(char *filename, int mode, struct formats *pf)
    * We use different a different permissions scheme if `cancreate' is
    * set.
    */
-  wmode = O_TRUNC;
-  if (stat(filename, &stbuf) < 0) {
-    if (!cancreate)
-      return (errno == ENOENT ? ENOTFOUND : EACCESS);
-    else {
-      if ((errno == ENOENT) && (mode != RRQ))
-	wmode |= O_CREAT;
-      else
-	return(EACCESS);
+  wmode = O_WRONLY |
+    (cancreate ? O_CREAT : 0) |
+    (unixperms ? O_TRUNC : 0);
+
+  fd = open(filename, mode == RRQ ? O_RDONLY : wmode, 0666);
+  if (fd < 0) {
+    switch (errno) {
+    case ENOENT:
+    case ENOTDIR:
+      return ENOTFOUND;
+    case ENOSPC:
+      return ENOSPACE;
+    case EEXIST:
+      return EEXISTS;
+    default:
+      return EACCESS;
     }
+  }
+
+  if ( fstat(fd, &stbuf) < 0 )
+    exit(EX_OSERR);		/* This shouldn't happen */
+
+  if (mode == RRQ) {
+    if ( !unixperms && (stbuf.st_mode & (S_IREAD >> 6)) == 0 )
+      return (EACCESS);
+    tsize = stbuf.st_size;
+    /* We don't know the tsize if conversion is needed */
+    tsize_ok = !pf->f_convert;
   } else {
-    if (mode == RRQ) {
-      if ((stbuf.st_mode&(S_IREAD >> 6)) == 0)
+    if ( !unixperms ) {
+      if ( (stbuf.st_mode & (S_IWRITE >> 6)) == 0 )
 	return (EACCESS);
-      tsize = stbuf.st_size;
-      /* We don't know the tsize if conversion is needed */
-      tsize_ok = !pf->f_convert;
-    } else {
-      if ((stbuf.st_mode&(S_IWRITE >> 6)) == 0)
-	return (EACCESS);
-      tsize = 0;
-      tsize_ok = 1;
+
+      /* We didn't get to truncate the file at open() time */
+#ifdef HAVE_FTRUNCATE
+      if ( ftruncate(fd, (off_t)0) )
+	return(EACCESS);
+#endif
     }
+    tsize = 0;
+    tsize_ok = 1;
   }
-  fd = open(filename, mode == RRQ ? O_RDONLY : (O_WRONLY|wmode), 0666);
-  if (fd < 0)
-    return (errno + 100);
-  /*
-   * If the file was created, set default permissions.
-   */
-  if ((wmode & O_CREAT) && fchmod(fd, 0666) < 0) {
-    int serrno = errno;
-    
-    close(fd);
-    unlink(filename);
-    
-    return (serrno + 100);
-  }
+
   file = fdopen(fd, (mode == RRQ)? "r":"w");
   if (file == NULL)
-    return (errno + 100);
+    exit(EX_OSERR);		/* Internal error */
+
   return (0);
 }
 
